export interface TLTWH {
	left: number;
	top: number;
	width: number;
	height: number;
}

export type TContent = {
	text?: string;
	image?: string;
};

export type THighlightContent = {
	content: TContent;
};

export type TComment = {
	text: string;
	emoji: string;
};
export type THighlightComment = {
	comment: TComment;
};

export type TNewHighlight = {
	position: TLTWH;
	pageNumber: number;
};

export type IHighlightType = 'table' | 'cell' | 'page' | 'document' | 'form';

export type THighlight = { id: string; color: string; type: IHighlightType } & TNewHighlight;

export type TVIEWPORT = {
	convertToPdfPoint: (x: number, y: number) => Array<number>;
	convertToViewportRectangle: (pdfRectangle: Array<number>) => Array<number>;
	width: number;
	height: number;
};

export type TPDFJSViewer = {
	_pages: Array<{ div: HTMLDivElement; id: number }>;
	container: HTMLDivElement;
	viewer: HTMLDivElement;
	getPageView: (page: number) => {
		textLayer: { textLayerDiv: HTMLDivElement };
		viewport: TVIEWPORT;
		div: HTMLDivElement;
		canvas: HTMLCanvasElement;
	};
	setDocument: (document: TPDFJSDocument) => Promise<void>;
	scrollPageIntoView: (options: { pageNumber: number; destArray: Array<any> }) => void;
	currentScaleValue: string;
};

export interface TPDFJSDocument {
	destroy: () => void;
	numPages: number;

	getPage(number: number): any;
}

export interface TPDFJSLinkService {
	setDocument: (document: any) => void;
	setViewer: (viewer: TPDFJSViewer) => void;
}

export interface TPDFJS {
	TextLayerBuilder: {
		prototype: {
			_bindMouse: () => void;
		};
	};
	PDFViewer: (options: any) => TPDFJSViewer;
	PDFLinkService: () => TPDFJSLinkService;
	getDocument: (url: string) => Promise<TPDFJSDocument>;
	disableWorker: boolean;
}

export type THighlightItem<THT> = Omit<THT, 'position'> & { position: TLTWH };

export type THighLightItemProps = {
	onClick?: (id: string) => void;
	onMouseOver?: () => void;
	onMouseOut?: () => void;
};

export interface PdfHighlighterProps<THT> {
	highlights: Array<THT>;
	pdfDocument: TPDFJSDocument;
	pdfScaleValue: string;
	removePageBorders?: boolean;
	onChangeActivePage(page: number): void;
	activePage: number;
	highlightAreaProps?: THighLightItemProps;
}

export interface SinglePdfRenderProps {
	pdfDocument: TPDFJSDocument;
	pageNumber: number;
	highlights?: Array<THighlight>;
	highlightAreaProps?: THighLightItemProps;
	activeHighlightId?: string;
	zoomValue?: number;
}

export interface PdfLoaderProps {
	data?: Blob;
	url?: string;
	beforeLoad: any;
	errorMessage?: any;
	children: (pdfDocument: TPDFJSDocument) => any;
	onError?: (error: Error) => void;
	cMapUrl?: string;
	cMapPacked?: boolean;
}
